@@ -1,5 +1,5 @@
 # JUBE Benchmarking Environment
-# Copyright (C) 2008-2021
+# Copyright (C) 2008-2022
 # Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre
 # http://www.fz-juelich.de/jsc/jube
 #
@@ -448,7 +448,9 @@ class Analyser(object):
                             else:
                                 match_dict[pattern.name]["sum2"] = match ** 2
                         except OverflowError:
-                            LOGGER.warning("Squared sum cannot be represented, numerical result out of range.")
+                            LOGGER.warning(
+                                "Squared sum cannot be represented, " +
+                                "numerical result out of range.")
                             match_dict[pattern.name]["sum2"] = math.nan
 
                     if "cnt" in match_dict[pattern.name]:
@@ -465,10 +467,10 @@ class Analyser(object):
                     if match_dict[pattern.name]["cnt"] > 1:
                         try:
                             match_dict[pattern.name]["std"] = math.sqrt(
-                                      (abs(match_dict[pattern.name]["sum2"] - 
-                                      (match_dict[pattern.name]["sum"] ** 2 /
+                                (abs(match_dict[pattern.name]["sum2"] -
+                                     (match_dict[pattern.name]["sum"] ** 2 /
                                       match_dict[pattern.name]["cnt"])) /
-                                      (match_dict[pattern.name]["cnt"] - 1)))
+                                 (match_dict[pattern.name]["cnt"] - 1)))
                         except OverflowError:
                             match_dict[pattern.name]["std"] = 0
                     else:
@@ -491,7 +493,8 @@ class Analyser(object):
         for pattern_name in match_dict:
             for option in match_dict[pattern_name]:
                 if option == "first":
-                    result_dict[pattern_name] = match_dict[pattern_name][option]
+                    result_dict[pattern_name] = \
+                        match_dict[pattern_name][option]
                 name = "{0}_{1}".format(pattern_name, option)
                 result_dict[name] = match_dict[pattern_name][option]
 
