@@ -1,5 +1,5 @@
 # JUBE Benchmarking Environment
-# Copyright (C) 2008-2021
+# Copyright (C) 2008-2022
 # Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre
 # http://www.fz-juelich.de/jsc/jube
 #
@@ -57,10 +57,10 @@ class YAML_Converter(object):
          "analyse": ["file"], "analyser": ["use", "analyse"],
          "fileset": ["link", "copy", "prepare"],
          "include-path": ["path"], "parameterset": ["parameter"],
-         "patternset": ["pattern"], "result": ["use", "table", "syslog"],
+         "patternset": ["pattern"], "result": ["use", "table", "syslog", "database"],
          "selection": ["not", "only", "tag"], "step": ["use", "do"],
          "substituteset": ["iofile", "sub"], "syslog": ["key"],
-         "table": ["column"]}
+         "table": ["column"], "database": ["key"]}
 
     def __init__(self, path, include_path=None, tags=None):
         self._path = path
@@ -107,9 +107,7 @@ class YAML_Converter(object):
                 yaml.load(file_handle.read(), Loader=yaml.Loader), xmltree)
             xml = jube2.util.output.element_tree_tostring(
                 xmltree, encoding="UTF-8")
-            dom = DOM.parseString(xml.encode('UTF-8'))
-            self._int_file.write(dom.toprettyxml(
-                indent="  ", encoding="UTF-8"))
+            self._int_file.write(xml.encode('UTF-8'))
         LOGGER.debug("  YAML Conversion finalized")
 
     def read(self):
@@ -129,6 +127,7 @@ class YAML_Converter(object):
         else:
             raise ValueError(("\"{0}\" not found in possible " +
                               "include pathes").format(filename))
+
         return file_path
 
     def __search_for_tags(self):
@@ -213,6 +212,8 @@ class YAML_Converter(object):
     @staticmethod
     def create_headtags(data, parent_node):
         """ Search for the headtags in given dictionary """
+        if type(data) is not dict:
+            data = {'benchmark': data}
         to_delete = list()
         for tag in data.keys():
             if type(data[tag]) is not list:
